
local assert, getmetatable, ipairs, pcall, setmetatable, tonumber, tostring = assert, getmetatable, ipairs, pcall, setmetatable, tonumber, tostring

local string              = require("string")
local table               = require("jive.utils.table")
local io                  = require("io")
local oo                  = require("loop.simple")
local log                 = require("jive.utils.log").logger("applets.misc")
local debug               = require("jive.utils.debug")

local Label               = require("jive.ui.Label")
local Applet              = require("jive.Applet")
local Window              = require("jive.ui.Window")
local Surface             = require("jive.ui.Surface")
local Icon                = require("jive.ui.Icon")
local Framework           = require("jive.ui.Framework")
local Popup               = require("jive.ui.Popup")
local Textarea            = require("jive.ui.Textarea")
local Textinput           = require("jive.ui.Textinput")

local socket          = require("socket")

local EVENT_WINDOW_POP = jive.ui.EVENT_WINDOW_POP
local EVENT_KEY_ALL    = jive.ui.EVENT_KEY_ALL
local EVENT_KEY_DOWN   = jive.ui.EVENT_KEY_DOWN
local EVENT_KEY_UP     = jive.ui.EVENT_KEY_UP
local EVENT_KEY_PRESS  = jive.ui.EVENT_KEY_PRESS
local EVENT_KEY_HOLD   = jive.ui.EVENT_KEY_HOLD
local EVENT_CONSUME    = jive.ui.EVENT_CONSUME
local EVENT_UNUSED     = jive.ui.EVENT_UNUSED
local EVENT_ACTION     = jive.ui.EVENT_ACTION
local EVENT_SCROLL     = jive.ui.EVENT_SCROLL
local KEY_FWD          = jive.ui.KEY_FWD
local KEY_REW          = jive.ui.KEY_REW
local KEY_HOME         = jive.ui.KEY_HOME
local KEY_PLAY         = jive.ui.KEY_PLAY
local KEY_ADD          = jive.ui.KEY_ADD
local KEY_BACK         = jive.ui.KEY_BACK
local KEY_PAUSE        = jive.ui.KEY_PAUSE
local KEY_VOLUME_DOWN  = jive.ui.KEY_VOLUME_DOWN
local KEY_VOLUME_UP    = jive.ui.KEY_VOLUME_UP
local KEY_GO           = jive.ui.KEY_GO
local WHEEL_CW         = jive.ui.KEY_DOWN
local WHEEL_CCW        = jive.ui.KEY_UP


local keyState = {}

local wheelState = 0

local keymap = {
    { key = KEY_FWD,   name = "scroll", extraTag = "direction", extraVal = "up" },
    { key = KEY_REW,   name = "scroll",extraTag = "direction", extraVal = "down" },
    { key = KEY_HOME,  name = "vudu" },
    { key = KEY_PLAY,  name = "play-pause", extraTag = "state", extraVal = "play" },
    { key = KEY_ADD,   name = "exit" },
    { key = KEY_BACK,  name = "back" },
    { key = KEY_PAUSE, name = "play-pause" },
    { key = KEY_VOLUME_DOWN, name = "ignore" },
    { key = KEY_VOLUME_UP,   name = "ignore" },
    { key = KEY_GO,    name = "select" },
}


module(...)
oo.class(_M, Applet)


function _addTag(tag, value)
   local newString = "<" .. tag .. ">" .. value .. "</" .. tag .. ">"
   return newString
end

function _createKey(name, extraName, extraValue)
   local newString = "<?xml version=\"1.0\"?>"
   newString = newString .. "<NetRemote><KeyEventRequest_1>"
   if name then
      newString = newString .. _addTag("name", name)
   else
      return ""
   end
   if extraName then
      newString = newString .. _addTag(extraName, extraValue)
   end
   newString = newString .. "<id>id-1</id>"
   newString = newString .. "</KeyEventRequest_1></NetRemote>"

   return newString
end

function _sendKey(name, extraName, extraValue)
   local sendString = _createKey(name, extraName, extraValue)

   tcpSocket:send(sendString)
end

function _waitForResponse()
   local success = false
   while true do
      r = tcpSocket:receive('*l')
      a,b = string.find(r, "<status>")
      if a then
     a = string.find(r, "OK", b)
     if a then
        success = true
     end
      end

      if r == "</NetRemote>" then
     break
      end
   end

   return success
end

function _scroll(self, dir)
   log:info("dir=", dir, "wheelState=", wheelState)

   wheelState = wheelState + dir

   if dir > 0 then
      _sendKey("scroll", "direction", "down")
   else
      _sendKey("scroll", "direction", "up")
   end
end

function _completedShow(self)
    local popup = Window("window")
    popup:addWidget(Textarea("textarea", "NetRemote Done"))

    self:tieWindow(popup)
    popup:showBriefly(2000, function() self.window:hideToTop() end)
end

function SetNetRemoteIp(self)
   local window = Window("window")
   self.window = window

   log:info("Set NetRemote IP")

   local settings = self:getSettings()
   local ipAddress = settings["_VUDU_IP"]
   if ipAddress == nil then
      ipAddress = "0.0.0.0"
   end

   local v = Textinput.ipAddressValue(ipAddress)
    local input = Textinput("textinput", v,
                function(_, value)
                    log:warn("Input " .. value:getValue())
                    settings["_VUDU_IP"] = value:getValue()

                    window:playSound("WINDOWSHOW")
                    window:hide(Window.transitionPushLeft)
                    return true
                end)

    local help = Textarea("help", "Set the VUDU NetRemote IP address.")

    window:addWidget(help)
    window:addWidget(input)

    self:tieAndShowWindow(window)
    log:info("ip address: " .. settings["_VUDU_IP"] )

    window:addListener(EVENT_WINDOW_POP,
        function()
            self:storeSettings()
        end
    )

   self:tieAndShowWindow(window)
   return window

end

function NetRemote(self)
   local window = Window("window")
   self.window = window

   self.background = Surface:loadImage("applets/NetRemote/NetRemote.png")

   self.icon = Icon("icon")
   window:addWidget(self.icon)
    window:addWidget(Textarea("help", "You are holding a VUDU NetRemote"))

   local w, h = Framework:getScreenSize()
   local srf  = Surface:newRGB(w, h)
   self.background:blit(srf, 0, 0)
   self.icon:setValue(srf)

   local settings = self:getSettings()
   local ipAddress = settings["_VUDU_IP"]
   if ipAddress == nil then
      ipAddress = '192.168.0.77'
   end
   log:info("Connecting to VUDU box: " .. ipAddress)
   tcpSocket = socket.tcp()
   local retval = tcpSocket:connect(ipAddress, 9876)
   if retval == nil then
      return window
   end

   window:addListener(EVENT_KEY_ALL | EVENT_SCROLL,
              function(event)
             local eventtype = event:getType()

             if eventtype then
                if eventtype == EVENT_SCROLL then
                   window:playSound("CLICK")
                   self:_scroll(event:getScroll())

                elseif eventtype | EVENT_KEY_ALL then
                   window:playSound("SELECT")

                   -- update key state
                   local keycode = event:getKeycode()

                   for i,k in ipairs(keymap) do
                  if keycode & k.key == k.key then
                     if eventtype == EVENT_KEY_DOWN then
                    if k.name == "exit" then
                       log:info("exit called")
                       _completedShow(self)
                    elseif k.name ~= "ignore" then
                       log:info("Sending key event ", k.name)
                       _sendKey(k.name, k.extraTag, k.extraVal)
                    end
                     end
                  end
                   end
                end

                return EVENT_CONSUME
             end
              end
           )

   self:tieAndShowWindow(window)
   return window
end


--[[

=head1 LICENSE

Copyright 2007 Logitech. All Rights Reserved.

This file is subject to the Logitech Public Source License Version 1.0. Please see the LICENCE file for details.

=cut
--]]
